function B = MMBSpec (BSpec, SFreq, BSpecDef)
% Check and fill in missing band specifications
%
% $Id: MMBSpec.m,v 1.6 2009/07/07 14:33:30 pkabal Exp $

% The input and output structures are indexed by band. Thus
% BSpec(1) has fields BSpec(1).Freq, BSpec(1).Weight, etc.

if (nargin < 3)
  BSpecDef = [];
end

% Set up defaults, establish missing fields
[BSpec, BSpecDef] = MM_BSpecDef (BSpec, BSpecDef);

% Check the frequencies
if (any (diff ([BSpec(:).Freq]) <= 0))
  error ('DFiltFIR: Frequencies not ordered');
end

% Error checks, band by band
NBand = length (BSpec);
for (i = 1:NBand)
  B(i) = MM_BandCheck (BSpec(i), SFreq, BSpecDef);
end

return

%--------------------
function [BSpec, BSpecDef] = MM_BSpecDef (BSpec, BSpecDef)

% Freq and Value fields must be present
if (~ isfield (BSpec, 'Freq'))
  error ('DFiltFIR: No frequencies specified');
end
if (~ isfield (BSpec, 'Value'))
  error ('DFiltFIR: No values specified');
end

% Set up defaults (BSpecDef), if not specified
% Create fields in BSpec and populate with default values
DefFields = {'Weight', 'LLimit', 'ULimit', 'ValueInt', 'WeightInt', ...
              'LLimitInt', 'ULimitInt'};
DefFieldVals = {1, -Inf, Inf, 'linear', 'linear', 'linear', 'linear'};
NDF = length (DefFields);
for (i = 1:NDF)
  Field = DefFields{i};
  if (~isfield (BSpecDef, Field))
    BSpecDef.(Field) = DefFieldVals{i};
  end
  if (~ isfield (BSpec, Field))
    [BSpec.(Field)] = deal (BSpecDef.(Field));
  end
end

% Check for spurious or misspelled field names
if (length (fieldnames (BSpec)) ~= 9 || ...
    length (fieldnames (BSpecDef)) ~= 7)
  error ('DFiltFIR: Invalid field name');
end

return

%---------------------
function B = MM_BandCheck (BSpec, SFreq, BSpecDef)
% The input and output structures are for a single band.

% Normalize the frequency
B.f = BSpec.Freq / SFreq;
NF = length (B.f);

% Error check
if (isempty (BSpec.Freq))
  error ('DFiltFIR: Missing band frequency');
end
if (any (B.f < 0) || any (B.f > 0.5))
  error ('DFiltFIR: Frequency out of range');
end

% Values
if (isempty (BSpec.Value))
  error ('DFiltFIR: Missing band value');
end
B.V = MM_FillV (BSpec.Value, NF);

% Weights
if (isempty (BSpec.Weight))
  error ('DFiltFIR: Missing band weight');
end
B.W = MM_FillV (BSpec.Weight, NF);
if (any (B.W <= 0))
  error ('DFiltFIR: Invalid weight');
end

% Limits
if (isempty (BSpec.LLimit))
  BSpec.LLimit = BSpecDef.LLimit;
end
B.LL = MM_FillV (BSpec.LLimit, NF);
if (isempty (BSpec.ULimit))
  BSpec.ULimit = BSpecDef.ULimit;
end
B.UL = MM_FillV (BSpec.ULimit, NF);
if (any (B.LL > B.V) || any (B.UL < B.V))
  error ('DFiltFIR: Invalid limits');
end

% Interpolation domains
B.VI = MM_IntDomain (BSpec.ValueInt);
B.WI = MM_IntDomain (BSpec.WeightInt);
B.LLI = MM_IntDomain (BSpec.LLimitInt);
B.ULI = MM_IntDomain (BSpec.ULimitInt);

return

%--------------------
function X = MM_FillV (X, N)

if (length (X) == 1)
  X(1:N) = X(1);
end
if (length (X) ~= N)
  error ('DFiltFIR: Invalid number of values')
end

return

%--------------------
function ID = MM_IntDomain (ID)

strarray = {'linear', 'log', 'sqrt'};
if (isempty (ID))
  ID = 'linear';
else
  i = strmatch(lower(ID), strarray);
  if (isempty (i))
    error ('DFiltFIR: Invalid interpolation domain');
  else
    ID = strarray{i};
  end
end

return
