function yi = IntMC (x, y, xi)
% Piecewise monotonic cubic interpolation.
%   yi: Interpolated values at xi
%
%   x:  Reference x values
%   y:  Reference y values
%   xi: Interpolation points
% This routine calculates interpolated values for a function defined by
% a table of reference data values. The interpolated values are found by
% passing a piecewise monotonic cubic interpolant through the reference
% data values. The interpolated values are monotonic between reference
% points and do not go outside the range of the bracketing reference data
% values.
%
% The procedure defines the cubic interpolant between any two adjacent
% reference points in terms of the values and the derivatives at the
% reference points. The derivatives at the reference points are determined
% by averaging the inverse slopes of the straight line segments joining the
% reference points to the left and right of a given reference point. These
% slope values are changed if necessary to guarantee that the cubic
% interpolants are monotonic between reference points. For instance, a
% reference point is a local extremum (the slopes of the straight line
% segments joining reference points on either side of it are of different
% sign), the slope at that reference point is set to zero. The slope of end 
% points is determined from the slopes of the two straight lines joining
% the reference points beside the end point.
%
% For values of xi outside the interval [x(1), x(N)], the corresponding
% value of yi is extrapolated using the cubic fit for the interval
% [x(1) x(2)] or [x(N-1) x(N)]. For two reference points, linear
% interpolation is used. For a single reference point, the output is equal
% to y at that point for all values of xi.
%
% Reference:
%     Subroutine PCHIM, F. N. Fritsch, Lawrence Livermore National
%     Laboratory.
%     F. N. Fritsch and J. Butland, "A Method for Constructing Local
%     Monotone Piecewise cubic interpolants", SIAM J. Sci. Stat. Comput.,
%     vol. 5, pp. 300-304, June 1984.

% $Id: IntMC.m,v 1.4 2009/06/05 15:48:29 pkabal Exp $

% Special case for 1 reference point
Nx = length(x);
if (Nx == 1)
  yi = y * ones(size(xi));
  return
end

% Quantize the xi, I is the index of the region
[temp, I] = histc(xi, x);

% Check for out-of-bounds xi values
I(xi < x(1)) = 1;
I(xi >= x(Nx)) = Nx-1;

% Region         Range
%   1     -Inf <= xi < x(2)
%   2     x(2) <= xi < x(3)
%  ...           ...
%  N-1  x(N-1) <= xi <= Inf

% Compute slopes at the reference points
d = SlopeMC(x, y);

% Interpolate
dxL = xi - x(I);
dxR = x(I+1) - xi;
dx = diff(x);
dxLN = dxL ./ dx(I);
dxRN = dxR ./ dx(I);
dxdxN = dxLN .* dxRN;

yi = dxdxN .* (d(I) .* dxR - d(I+1) .* dxL) + ...
          dxRN .* (2 * dxdxN + dxRN) .* y(I) + ...
          dxLN .* (2 * dxdxN + dxLN) .* y(I+1);

return
