function yi = IntMCI (x, y, xi)
% Piecewise monotonic cubic interpolation.
%   yi: Interpolated values at xi
%
%   x:  Reference x values
%   y:  Reference y values
%   xi: Interpolation points
%
% This routine is a front end to routine IntMC. It handles the case of some
% reference values being infinite. The reference points are divided into
% segments of containing only finite y values. Interpolation is done on
% these segments individually. The segments containing infinite values will
% return infinite values (of the appropriate sign) for the xi values which
% lie in them.

% $Id: IntMCI.m,v 1.1 2009/06/16 16:33:21 pkabal Exp $

% The input vector y can contain infinite values. Consider the reference
% values,
%   y = [0 1 Inf 1 0]
% An interpolant for the data has a linear segment between the first two
% reference points. The second reference point is included as the end of
% the linear segment. Immediately after the second reference point, the
% interpolated function jumps to infinity. The function falls back down
% just before the fourth referencepoint. The interpolated function between
% the fourth and fifth points is a linear segment.
%
% For this example we create three segments.
%  1) [y(1) y(2)]  % includes the end points
%  2) (y(2) y(4))  % end points are not included
%  3) [y(4) y(5)]  % includes the end points
% Interpolation will be applied independently in these sections. The
% segment containing infinite valued point will have a constant value set
% to infinity.

% Initialize yi
yi = NaN(size(xi));

% Look for segments that need to be interpolated
% - Find the indices of the non-infinite elements
% - Look for runs of indices: breaks in the runs are indicated when the
%   diff of the indices is larger than one. The index of diff will point
%   to the end of a run. By adding a trailing Inf to the array of indices,
%   we can get a pointer to the end of the last run.
% - The beginning of a run can be found by prepending a -Inf to the
%   beginning of the array of indices and looking for differences in
%   indices which are larger than 1.
Fy = find(isfinite(y));         % Indices
StF = Fy(diff([-Inf Fy]) > 1);  % Difference of indices; empty for empty Fy
EnF = Fy(diff([Fy Inf]) > 1);
NsF = length(StF);              % Number of segments

Nx = length(x);

% Delimit segments that include infinities
% Initially set up segments before, between, and after the segments
% of finite values. Remove the initial and/or the final segment if
% necessary.
if (isempty(StF))
  StI = 1;
  EnI = Nx;
else
  StI = [1 EnF];
  EnI = [StF Nx];
  if (StF(1) == 1)
    StI(1) = [];
    EnI(1) = [];
  end
  if (EnF(end) == Nx)
    StI(end) = [];
    EnI(end) = [];
  end
end
NsI = length(StI);

% Handle infinities segment by segment
for (i = 1:NsI)
  is = StI(i);
  ie = EnI(i);
  xst = x(is);
  if (is == 1)
    xst = -Inf;
  end
  xfn = x(ie);
  if (ie == Nx)
    xfn = Inf;
  end
  I = find(xi > xst & xi < xfn);
  yi(I) = MC_Inf(x(is:ie), y(is:ie), xi(I));
end

% Interpolate segment by segment
for (i = 1:NsF)
  is = StF(i);
  ie = EnF(i);
  xst = x(is);
  if (is == 1)
    xst = -Inf;    % Extrapolation for xi < x(1)
  end
  xfn = x(ie);
  if (ie == Nx)
    xfn = Inf;     % Extrapolation for xi > x(Nx)
  end
  I = find(xi >= xst & xi <= xfn);
  yi(I) = IntMC(x(is:ie), y(is:ie), xi(I));
end

return

% ----- -----
function yi = MC_Inf (x, y, xi)
% Handle segments containing infinite values
% - For intermediate segments, the initial value and final value
%   are finite. Other values are infinite.
% - For a starting segment, the initial finite value is missing
% - For an end segment, the final finite value is missing
% - For a change of sign, between infinite values, the value at exactly
%   the midpoint will be set to zero

% Special case for 1 reference point
Nx = length(x);
if (Nx == 1)
  yi = y * ones(size(xi));
  return
end

% Quantize the xi, I is the index of the region
[temp, I] = histc(xi, x);

% Check for out-of-bounds xi values
I(xi < x(1)) = 1;
I(xi >= x(Nx)) = Nx-1;

% Region         Range
%   1     -Inf <= xi < x(2)
%   2     x(2) <= xi < x(3)
%  ...           ...
%  N-1  x(N-1) <= xi <= Inf

% For the case that the end points of the segment are finite, we force
% the interior infinities on them. (If the end points are finite, then
% the values xi must lie strictly inside the segment.)
if (isfinite(y(1)))
  y(1) = y(2);
end
if (isfinite(y(Nx)))
  y(Nx) = y(Nx-1);
end

yi = NaN(size(xi));

d = 2*xi - (x(I) + x(I+1));  % 2 * distance from mid point of a region

% Segregate xi's
K0 = find(d == 0);  % Midway between x values
KP = find(d > 0);   % Nearer right x value
KM = find(d <= 0);  % Nearer left x value

% For xi values midway between x values, find those for which the
% bracketing values have the same sign
K0D = K0(sign(y(I(K0))) ~= sign(y(I(K0)+1)));

yi(KP) = y(I(KP)+1);
yi(KM) = y(I(KM));
yi(K0D) = 0;   % Overlay mid values for the case of different signs

return
