function alpha = MMcosCof (xs, xf, Ex)
% Given the sample values for a polynomial, this routine returns the
% coefficients of the sum of cosines expansion. The expansion is of
% the form
%
%          NCof-1
%   G(w) =  SUM  a(n) cos(wn) , where NCof = NExt-1.
%           n=0
%
% The input values are the values of G(w), y(i)=G(w(i)) and x(i)=cos(w(i)).
%
% alpha: Returned array of coefficients for the polynomial that passes
%     through the given values.
%
% xs: Starting abscissa limit. This value is in the interval [max(x),+1].
%     Together with xf it defines the interval over which the design was
%     specified.
% xf: Last abscissa limit. This value is in the interval [-1,min(x)].
%     Together with xs it defines the interval over which the design was
%     specified.
% Ex: Structure with the extrema
%     Ex.x: Array of abscissa values at the extremal points.
%     Ex.y: Array of ordinate values at the extremal points.
%          The NExt extrema are constrained in such a way as to allow a
%          NExt-1 term polynomial (normally one term too few) to pass
%          through the NExt points.

% $Id: MMcosCof.m,v 1.4 2009/06/03 20:41:15 pkabal Exp $

EPS = 0.05;

% Set up the interpolation structure
P.x = Ex.x;
P.y = Ex.y;

% Generate the Lagrange interpolation coefficients:
% Note that there are NExt values given. In general an NExt term polynomial
% is required to pass throught these points.  However, it is assumed that
% the values are constrainted such that a NExt-1 term polynomial can pass
% through the NExt points. We could just use a subset of NExt-1 values to
% do the Lagrange interpolation. However, we opt here to use NExt values as
% was done at the design stage. The frequency response will then be NExt-1
% samples from this (NExt term) polynomial.
P.a = MMintCof (P.x);

% If the band specifications do not extend from 0 to 1/2 (normalized Hz) or
% +1 to -1 (x = cos(2 Pi f) domain), the frequency response is not sampled
% at the edges. The full band samples are x(k) = cos (2 Pi k/M). For
% partial band sampling, the samples are taken at c*cos(2 Pi k/M) + d.
% It is not clear how much this partial band sampling helps - nearly all
% filter design problems invoke full band sampling.
if (xs - xf > 2 - EPS)
  xs = 1;
  xf = -1;
end
c = 0.5 * (xs - xf);	% Note: xs > xf
d = 0.5 * (xf + xs);

% Sample the response at Npar equally spaced values
NExt = length (P.x);
NPar = NExt - 1;
G = MM_FResp (c, d, NPar, P);

% Evaluate the inverse DFT to find the cosine series coefficients
a = MM_invDFT (G);

% The inverse discrete Fourier transform gives the coefficients a(n) of a
% Chebyshev series expansion,
%
%          N-1                N-1
%   P(x) = SUM a(n) cos(wn) = SUM a(n) T(n,x) .
%          n=0                n=0
%
% The Chebyshev polynomials are polynomials in x=cos(w). However what we
% really want are the coefficients of the expansion in y=cos(u),
%
%         N-1                N-1
%  G(u) = SUM b(n) cos(un) = SUM b(n) T(n,y).
%         n=0                n=0

% Routine MMtrCheb finds the coefficients b(n), given the coefficients a(n)
% and the transformation y=cx+d. For full band sampling, MMtrCheb copies a
% to alpha.

alpha = MMtrCheb (a, c, d);

return

%--------------------
function G = MM_FResp (c, d, NPar, P)

% Evaluate the polynomial using MMintVal at equally spaced frequency
% intervals. While normalized frequency goes from 0 to 0.5, x goes from
% +1 to -1.

FSH	= 1e-10;

M = 2 * NPar - 1;
n = 1;
Nx = length (P.x);
G = zeros (1, NPar);
for (k = 1:NPar)

  xt = c * cos ((2 * pi * (k-1)) / M) + d;

  % If the abscissa value lies close to one of the grid values, use the
  % corresponding ordinate value to avoid a divide by zero in MMintVal.
  % The forbidden region around the grid values can be very small. In the
  % barycentric form of Lagrange interpolation, inaccuracies tend to
  % cancel when near the grid value.
  while (xt <= P.x(n) - FSH && n + 1 <= Nx)
    n = n + 1;
  end

  if (abs (xt - P.x(n)) < FSH)
    G(k) = P.y(n);
  else
    G(k) = MMintVal (xt, P);
  end

end

return

%--------------------
function a = MM_invDFT (G)
% Inverse Discrete Fourier Transform for a symmetric sequence

% The frequency response is
%
%           N-1                 M-1
%    G(w) = SUM a(n) cos(wn)  = SUM g(n) exp(jwn)
%           n=0                 n=0
%
% where M = 2N-1,
%    g(0) = a(0),                      a(0) = g(0)
%    g(n) = a(n)/2  for 1 <= n < N,    a(n) = 2 g(n)  for 1 <= n < N
%    g(n) = g(M-n)  for N <= n < M.
%
% Let G(k) = G(2 Pi k/M), the DFT of {g(0),g(1),...,g(M-1)}.
% G(k) = G(M-k). The inverse DFT gives
%
%          1           N-1
%   g(0) = - [G(0) + 2 SUM G(k)],
%          M           k=1
%
%          1            N-1
%   g(n) = - [ G(0) + 2 SUM G(k) cos(2 Pi kn/M) ],  1 <= n < N.
%          M            k=1

NPar = length (G);
M = 2 * NPar - 1;
g = zeros (1, NPar);
for (i = 0:NPar-1)
  s = sum (G .* cos ((2 * pi * (0:NPar-1) * i) / M));
  g(i+1) = (2 * s - G(1)) / M;
end
a(1) = g(1);
a(2:NPar) = 2 * g(2:NPar);

return
