function [Ex, ier] = MMdesRemez (NExt, G)
% This routine finds a minimum maximum deviation approximation to a
% function, subject to constraints of the allowed deviation. The function
% is specified by the desired function values and weights on a dense grid.
% The algorithm minimizes the Chebyshev error by determining the best
% locations of the extremal points (points of maximum error). The returned
% values are the extremal points of the approximation.
%
% Ex: Structure with the extrema
%     Ex.x: Array of abscissa values at the extremal points.
%           Each x(i) takes on a unique value from G.x(k).
%     Ex.y: Array of ordinate values at the extremal points.
%          The NExt extrema are constrained in such a way as to allow a
%          NExt-1 term polynomial (normally one term too few) to pass
%          through the NExt points.
%     Ex.EType: Type of extremum,
%           -2 - Lower constraint limit
%           -1 - Lower ripple
%           +1 - Upper ripple
%           +2 - Upper constraint limit
%     Ex.dev: Resultant deviation (relative to unity weighting)
% ier: Error parameter coded as follows.
%     0  - No error
%     1  - Remez algorithm failed to converge, the deviation is not
%          monotonically increasing
%     2  - Remez algorithm failed to converge, too many iterations
%     3  - Constraints too tight for the given filter order
%     For these cases, this routine returns coefficient values. For all
%     other errors, an error message is printed and execution is halted.
%
% NExt: Number of extrema in the approximation
% G: Structure with the arrays specifying the filter specifications on the
%    grid

% $Id: MMdesRemez.m,v 1.4 2009/06/03 20:40:52 pkabal Exp $

MAXITER = 40;

devl = -1;

% Set up the alternating limit array
[Ext, EType] = MM_initExt (NExt, G);

% ********************
for (iter = 1:MAXITER)

  % Store the abscissa values for the Lagrange interpolation
  P.x = G.x(Ext);

  % Find the Lagrange interpolation coefficients to interpolate NExt values 
  P.a = MMintCof (P.x);

  % Find the ordinates at the putative extremal points
  % For constained designs, adjust EType at places where the limits are
  % violated.
  [devn, P.y, EType, ier] = MM_CalcOrd (Ext, EType, P, G);
  if (ier ~= 0)
    break;
  end
  
  % Check the convergence
  if (devn < devl)
    ier = 1;
    disp ('DFiltFIR - Failed to converge (deviation non-increasing)');
    break;
  end
  devl = devn;

  %---------------
  % Remez exchange algorithm
  [NChange, Ext, EType] = MMexchExt (devn, Ext, EType, P, G);
  if (NChange == 0)
    break;
  end
  %---------------

end
% ********************
% Failed to converge
if (iter > MAXITER)
  ier = 2;
  disp ('DFiltFIR - Failed to converge (too many iterations)');
end

% Return information about the extrema
Ex.x = P.x;
Ex.y = P.y;
Ex.EType = EType;
Ex.dev = devn;

return

%--------------------
function [Ext, EType] = MM_initExt (NExt, G)
% Initialize the extremal values and signs

E_UDEV = 1;
E_LDEV = -1;

% Initial guess for the extremal frequencies (equally spaced  along the
% frequency grid)
NGrid = length (G.x);
dens = (NGrid - 1) / max (NExt - 1, 1);
Ext = round ((0:NExt-1) * dens) + 1;

% Set up the signs of the deviations - alternating 
% The initial setting might have to be inverted
EType(1:2:NExt) = E_LDEV;
EType(2:2:NExt) = E_UDEV;

% Store the abscissa values for the Lagrange interpolation
% Find the Lagrange interpolation coefficients to interpolate NExt values
% Calculate the deviation
x = G.x(Ext);
a = MMintCof (x);
devn = MMmmDev (a, G, Ext, EType);

% Invert the signs of the deviations if necessary
if (devn < 0)
  EType = -EType;
end

return

%--------------------
function [devn, y, EType, ier] = MM_CalcOrd (Ext, EType, P, G)
% Calculate the ordinate values, deviation, and change EType for
% for extrema which violate the limits.

MAXEADJ	= 4;

E_ULIM = 2;
E_UDEV = 1;
E_LDEV = -1;
E_LLIM = -2;

for (ie = 1:MAXEADJ)

  % Calculate the deviation:
  % Given a set of NExt distinct points, an NExt-1 coefficient polynomial
  % can give an error curve which exhibits equal amplitude alternations (in
  % a weighted sense) about the desired value. In such a formulation, the
  % free parameters are the NExt-1 coefficients and the deviation giving a
  % total of NExt linear equations.
  [devn, ier] = MMmmDev (P.a, G, Ext, EType);
  if (ier ~= 0)
    break;
  end
  if (devn < 0)
    error ('DFiltFIR: Error deviation has changed sign');
  end

  % Calculate the ordinates for the Lagrange interpolation. The polynomial
  % that passes through the NExt alternations with deviation as calculated
  % above has only NExt-1 terms. However, if we keep the ordinate values
  % for the NExt term interpolation at the NExt alternation values, the
  % Lagrange coefficients for the NExt term case can be used to interpolate
  % the NExt-1 term polynomial.

  %----------
  NChange = 0;
  NExt = length (Ext);
  y = zeros (1, NExt);
  for (i = 1:NExt)
    % EType[i] represents the type of deviation as determined from the
    % previous iteration (and set to E_UDEV or E_LDEV prior to the first
    % iteration). The deviation is calculated using MMmmDev. Using this
    % deviation value, new ordinate values are set. These new ordinate
    % values may change an ordinary extremum to a constrained extremum.
    % This loop recalculates the deviation if EType[i] changes.
    k = Ext(i);
    if (EType(i) == E_LDEV)
      y(i) = G.V(k) - devn / G.W(k);
      if (y(i) < G.LL(k))
        EType(i) = E_LLIM;
        NChange = NChange + 1;
      end
    elseif (EType(i) == E_UDEV)
      y(i) = G.V(k) + devn / G.W(k);
      if (y(i) > G.UL(k))
        EType(i) = E_ULIM;
        NChange = NChange + 1;
      end
    elseif (EType(i) == E_LLIM)
      y(i) = G.LL(k);
    elseif (EType(i) == E_ULIM)
      y(i) = G.UL(k);
    end
  end
  %----------
  if (NChange == 0)
    break;
  end

end
if (ie > MAXEADJ)
  error ('DFiltFIR: Extremum adjustment failed: constraints too tight');
end

return
