function [NChange, Ext, EType] = MMexchExt (dev, Ext, EType, P, G)
% The routine finds the locations of the extrema of the error function for
% a constrained weighted minimax polynomial fit to a tabulated function.
% This routine is intended to be used as part of a Remez exchange
% algorithm. The desired values are specified on a dense grid of points.
% Given an initial guess at the locations of a set of extremal points with
% alternating errors, this routine changes the locations of the extrema to
% coincide with the local maxima on the grid.
%
% NChange: Number of extrema changed. A zero value indicates that the Remez
%     exchange algorithm has converged.
% Ext: Indices of the extremal points. These are the updated values.
% EType: Type of extremum for each of the extremal points in array Ext.
%
% dev: Deviation at the design points. This is the value of the deviation
%     as determined by MMmmDev.
% Ext: Indices of the extremal points. The given input values are used as
%     starting points.
% EType: Type of extremum for each of the extremal points in array Ext.
%      -2 - Lower constraint limit
%      -1 - Lower ripple
%      +1 - Upper ripple
%      +2 - Upper constraint limit
% P: Structure containing the array of Lagrange interpolation coefficients
%     and the corresponding abscissa and ordinate values. These values
%     together specify the polynomial that passes through the points
%     specified.
% G: Structure with the arrays of specifications on a dense grid of points.

EPS	= 1e-5;

NExt = length (P.x);
NGrid = length (G.x);
NChange = 0;
kext0 = Ext(1);
kextN = Ext(end);

% *************** Search for the NExt largest alternating extrema
% klow, kup form boundaries, search in klow+1,...,kup-1
klow = 0;
for (i = 1:NExt)

  k = Ext(i);
  [err, EType(i)] = MM_wgtErr (P.y(i), dev, EType(i), ...
                               G.V(k), G.W(k), G.LL(k), G.UL(k));

  if (i == NExt)
    kup = NGrid + 1;
  else
    kup = Ext(i+1);
  end 

  while (1)     % one-trip loop to allow breaks

    % Look for a local maximum by increasing the grid index
    [kext, err, EType(i)] = MM_errMax (k+1:kup-1, 0, err, dev, ...
                                       EType(i), P, G);
    if (kext >= 0) 
      break
    end

    % Look for a local maximum by decreasing the grid index
    [kext, err, EType(i)] = MM_errMax (k-1:-1:klow+1, 0, err, dev, ...
                                       EType(i), P, G);
    if (kext >= 0) 
      break
    end

    % Keep looking for a local maximum, decreasing the grid index
    if (NChange == 0)
      [kext, err, EType(i)] = MM_errMax (k-2:-1:klow+1, 1, err, dev, ...
                                         EType(i), P, G);
      if (kext >= 0) 
        break;
      end
      
      % Keep looking for a local maximum, increasing the grid index
      [kext, err, EType(i)] = MM_errMax (k+2:kup-1, 1, err, dev, ...
                                         EType(i), P, G);
      if (kext >= 0) 
        break;
      end
    end
    
    break;   % one-trip only
  end

  % Update the extremum position
  if (kext >= 0)
    klow = max (kext, k);
    Ext(i) = kext;
    NChange = NChange + 1;
  else
    klow = k;
  end

  % Save the error for the first extremum
  if (i == 1)
    err0 = err;
  end
end

% Save the error for the last extremum
errN = err;

% *************** Endpoints search
% Search below the lower endpoint for a local maximum with an error larger
% than the error found at the upper endpoint
err = errN * (1 + EPS);
kfn = min (kext0, Ext(1)) - 1;
[kextB, err, ETypeB] = MM_errMax (1:kfn, 1, err, dev, -EType(1), P, G);
if (kextB >= 0)
  err0B = max (err0, err);
else
  err0B = err0;
end

% Search above the upper endpoint for a local maximum with an error larger
% than the error found at or below the lower endpoint
err = err0B * (1 + EPS);
kfn = max (kextN, Ext(NExt-1)) + 1;
[kextT, err, ETypeT] = MM_errMax (NGrid-1:-1:kfn, 1, err, dev, ...
                                  -EType(NExt), P, G);

if (kextT >= 0)
  % Found a local maximum above the former upper endpoint
  Ext = [Ext(2:end) kextT];
  EType = [EType(2:end) ETypeT];
  NChange = NChange + 1;
elseif (kextB >= 0)
  % Found a local maximum below the former lower endpoint
  Ext = [kextB Ext(1:end-1)];
  EType = [ETypeB EType(1:end-1)];
  NChange = NChange + 1;
end

return

%---------------------
function [kext, err, EType] = MM_errMax (krange, cont, err, dev, ...
                                         EType, P, G)
% This routine looks for a local maximum in the error curve. The routine
% checks a range of values on the approximating grid.
%
% kext: Position of the extremum found (in the range kst to kfn). This
%    value is set to -1 if no extremum with error larger than the initial
%    value of err is found.
% err: Largest error found.
% EType: Updated type of extremum.
%
% krange: Search indices
% cont: Search mode
%     0 - Continue searching as long as the error is increasing. If the
%         first probe is not successful, the search is terminated.
%     1 - Continue searching until a local maximum is found or the search
%         range is exhausted
% err: Largest error found so far.
% dev: Deviation at the design points
% EType: Type of extremum.
%     -2 - Lower constraint limit
%     -1 - Lower ripple
%     +1 - Upper ripple
%     +2 - Upper constraint limit
% P: Structure containing the array of Lagrange interpolation coefficients
%    and the corresponding abscissa and ordinate values. These values
%    together specify the polynomial that passes through the points
%    specified.
% G: Structure with the arrays specifying the specifications on a dense
%    grid of points

% $Id: MMexchExt.m,v 1.3 2009/06/03 20:18:57 pkabal Exp $

% For efficiency, separate code for upward and downward extrema
% The error calculation is the same as in MM_wgtErr
%------ Upward extremum
if (EType > 0)

  E_ULIM = 2;
  E_UDEV = 1;

  kext = -1;
  for (k = krange)

    % Calculate the ordinate values at G.x(k)
    y = MMintVal (G.x(k), P);

    % Calculate the weighted error, see if the error has increased
    % Upward peak
    errt = G.W(k) * (y - G.V(k));
    ET = E_UDEV;
    errL = (y - G.UL(k)) + dev;
    if (errL > errt)
      ET = E_ULIM;
      errt = errL;
    end

    % Use this grid position if the error increased
    if (errt > err)
      kext = k;
      EType = ET;
      err = errt;
    elseif (kext >= 0 || ~cont)
      % Did not find a better value this time, but found one earlier
      % (kext >= 0)
      break
    end
  end

%------ Downward extremum
else

  E_LDEV = -1;
  E_LLIM = -2;

  kext = -1;
  for (k = krange)

    % Calculate the ordinate values at G.x(k)
    y = MMintVal (G.x(k), P);

    % Calculate the weighted error, see if the error has increased   
    % Downward deviation
    errt = G.W(k) * (G.V(k) - y);
    ET = E_LDEV;
    errL = (G.LL(k) - y) + dev;
    if (errL > errt)
      ET = E_LLIM;
      errt = errL;
    end

    % Use this grid position if the error increased
    if (errt > err)
      kext = k;
      EType = ET;
      err = errt;
    elseif (kext >= 0 || ~cont)
      % Did not find a better value this time, but found one earlier
      % (kext >= 0)
      break
    end
  end

end
%------

return

%--------------------
function [err, EType] = MM_wgtErr (y, dev, EType, V, W, LL, UL)
% This routine calculates the weighted error for a given ordinate value.
% The errors for values which correspond to values which should lie above
% the desired value are calculated as
%   Eu = W*(y-V)     - ordinary value
%   EU = (y-UL)+dev  - value violating the constraints
% Similar expressions are applied for values which should lie below the
% desired value. The first formula is used, unless the value y violates
% the constraints. As the weighted error converges to become minimax, the
% ordinary extremal points have weighted deviation equal to dev. With this
% error expression, the extrema which approach the constraints also give
% errors which approach the value dev.
%
% The input parameter EType determines whether the value should lie above
% or below the desired value. Both Eu and EU are calculated. If EU > Eu,
% the value of EU is used. The value of EType is set according to which of
% Eu or EU is used.

if (EType > 0)
  E_ULIM = 2;
  E_UDEV = 1;

  % Upward peak
  err = W * (y - V);
  EType = E_UDEV;
  errL = (y - UL) + dev;
  if (errL > err)
    EType = E_ULIM;
    err = errL;
  end

else
  E_LDEV = -1;
  E_LLIM = -2;

  % Downward deviation
  err = W * (V - y);
  EType = E_LDEV;
  errL = (LL - y) + dev;
  if (errL > err)
    EType = E_LLIM;
    err = errL;
  end

end

return
