function h = MMfiltCof (FCase, alpha)
% This routine finds the coefficents of a linear phase FIR filter given the
% coefficients of a canonical response in terms of a cosine expansion. Four
% different cases are considered (number of coefficients even or odd, and h
% symmetric or anti-symmetric).
%
% h: Array of resultant filter coefficients
%
% Fcase: Filter case.
%    1 - odd number of coefficients, bandpass filter
%    2 - even number of coefficients, bandpass filter
%    3 - odd number of coefficients, differentiator or Hilbert transform
%    4 - even number of coefficients, differentiator or Hilbert transform
% alpha: Coefficients of the canonical response. The number of coefficients
%    M is uniquely related to the number of filter coefficients.

% $Id: MMfiltCof.m,v 1.2 2009/06/03 20:18:04 pkabal Exp $

% See MMmodSpec for the details of the relationships, H(z) = P(z) B(z).

%===========================
% Here we are given the a(n) coefficients. This is a one-sided set of
% coefficients (M values). First form the symmetric two-sided polynomial
% B'(z),   B'(z) = 0.5 (A(z) + A(1/z)).
% Note the overlapping of the zero'th coefficient. B'(z) has K=2M-1
% coefficients. Now shift B'(z),
%   B(z) = z^(-(M-1)) B'(z).
% Now form H(z),
%   H(z) = P(z) B(z).
% The numbers of coefficients are related as,
%   Nh = Np + Nb - 1
%      = Np + (2M-1) - 1.

% Fill in B(z) - B(z) is symmetric with an odd number of coefficients
% [ a[M-1], ... , a[1], a[0],    0, ... ,     0 ]
% [      0, ... ,    0, a[0], a[1], ..., a[M-1] ]
M = length (alpha);
b = 0.5 * ([fliplr(alpha) zeros(1,M-1)] + [zeros(1,M-1), alpha]);

% Convolve with P(z)
switch FCase
  case 1
    p = 1;
  case 2
    p = [ 1 1 ];
  case 3
    p = [ 1 0 -1 ];
  case 4
    p = [ 1 -1 ];
end
h = conv (p, b);

% Fix up the coefficients so that they are exactly symmetrical 
if (FCase == 1 || FCase == 2)
  h = 0.5 * (h + fliplr (h));
else
  h = 0.5 * (h - fliplr (h));
end

return
