function GB = MMgrid (FCase, NPar, BSpec, GridD)
% Given the input band specifications, this routine generates the
% specifications on a dense grid of frequency points.

% $Id: MMgrid.m,v 1.7 2009/07/07 14:37:16 pkabal Exp $

NGRID_MIN = 1000;
GRIDD_DEF = 16;

% Find the nominal number of grid points; the actual number will be
% higher since a higher density is used near band edges
if (isnan (GridD))
  NGrid = floor (max (NGRID_MIN, NPar * GRIDD_DEF));
else
  NGrid = floor (NPar * GridD);  % Nominal number of grid points
end
GridD = NGrid / NPar;      % Revised grid density

% Find the nominal grid spacing in Hz
Fext = MM_Fext (BSpec);    % Frequency extent of the bands
delf = Fext / NGrid;       % Hz per point
FI = 1.5 * GridD * delf;   % Band edge extent for better grid density

% Fill in the grid points
NBand = length (BSpec);
FGrid = cell (NBand, 1);
for (k = 1:NBand)
  Fw = MM_warpPar (BSpec(k).f, FI);
  FGrid{k}= MM_BandGrid (BSpec(k).f, Fw, delf, FCase);
end

% Generate the interpolated values, weights and limits at the frequency
% points determined above
B = MM_GridVal (BSpec, FGrid);

% Concatenate the band frequencies and values
GB.FGrid = [B(:).FGrid];
GB.V = [B(:).V];
GB.W = [B(:).W];
GB.LL = [B(:).LL];
GB.UL = [B(:).UL];

return

%--------------------
function FGrid = MM_BandGrid (f, FWarp, delf, FCase)
% This routine generates the grid of frequency points. These will
% eventually be transformed to x = cos(2*Pi*f). This means that points
% close to 0 and 0.5 will be more closely spaced. (For double precision,
% frequency values less than about 1e-8, all map to 1.) It is desirable to
% have more frequency points near band edges. However, the outside band
% edges (generally near 0 and 0.5) do not have adjacent bands. Band edges
% at exactly 0 and 0.5 (as occur in most filter designs) are really
% interior points of a concatenated band when viewed on a two-sided,
% periodic frequency scale.
%
% Bands are divided into subbands defined by the regions between the
% specified frequency points. The dense grid on a subband is such that the
% first and last points in the dense grid coincide with the ends of the
% subband.

% First point (and only point for 1-point bands)
FGrid = MM_SubbandGrid (f(1), f(1), FWarp, delf);

NF = length (f);
for (i = 2:NF)
  % Fill in the frequency values for the subband
  % The first point overlays the last point from the previous subband
  FG = MM_SubbandGrid (f(i-1), f(i), FWarp, delf);
  FGrid = [FGrid(1:end-1), FG];
end

% Remove one or both of the end points for cases for which the response
% at f=0 or f=0.5 is constrained
if ((FCase == 2 || FCase == 3) && FGrid(end) == 0.5)
  FGrid(end) = [];
elseif ((FCase == 3 || FCase == 4) && FGrid(1) == 0)
  FGrid(1) = [];
end

% Check for non-distinct points
if (isempty (FGrid))
  error('DFiltFIR: Invalid single point band location');
elseif (any (diff (FGrid) <= 0))
  error ('DFiltFIR: Grid spacing too small or invalid single point');
end

return

%--------------------
function FGrid = MM_SubbandGrid (fl, fu, Fw, delf)

gl = MM_warpInv (fl, Fw);
gu = MM_warpInv (fu, Fw);

% Choose the number of points to guarantee that the step size is at most
% delg
delg = delf / Fw.S;
N = ceil ((gu - gl) / delg) + 1;
FGrid = MM_Warp (linspace (gl, gu, N), Fw);

return

%---------------
function Fext = MM_Fext (BSpec)
% Find the frequency extent of the bands.

NBand = length (BSpec);
Fext = 0;
for (k = 1:NBand)
  Fext = Fext + (BSpec(k).f(end) - BSpec(k).f(1));
end

return

%---------------
function Fw = MM_warpPar (f, FI)
% Frequency warping functions
%
% The warping maps g->f or vice-versa. The piecewise linear mapping is
% defined by four points. The mapping is monotonic and so is invertible.
%   g[0]  =   f[0]
%   g[1] <--> f[1]
%   g[2] <--> f[2]
%   g[3]  =   f[3]
%
%  f[3]                        x
%                           o
%                        o
%  f[2]               x
%                    o
%                   o
%                  o
%                 o
%  f[1]          x
%             o
%          o
%  f[0] x--------|----|--------|
%     g[0]     g[1] g[2]     g[3]
%
% The mapping is determined from the intermediate frequency values
% (f[1], f[2]) and the ratios of the slopes of the segments
% (R[0], R[1], R[2]). The ratios of the slopes are specified as (1 2 1),
% i.e. the slope in the middle is twice the slopes at the ends.
%   f[1]-f[0]            f[2]-f[1]            f[3]-f[2]
%   --------- = R[0] S,  --------- = R[1] S,  --------- = R[2] S
%   g[1]-g[0]            g[2]-g[1]            g[3]-g[2]
% The value of S is determined from these relationships,
%   S (g[3]-g[0]) = (f[1]-f[0])/R[0] + (f[2]-f[1])/R[1] + (f[3]-f[2])/R[2]
% Then the values for g[1] and g[2] can be determined,
%   g[1] = (f[1]-f[0]) / (R[0] S) + g[0],
%   g[2] = (f[2]-f[1]) / (R[1] S) + g[1].

% Ratios of slopes of the mapping from g to f: the slope at the edges of
% the band is half the slope in the middle of the band. For uniform
% step sizes in g, this gives step sizes in f at the edges of the band
% which are half the step sizes near the middle of the band.
R = [1 2 1];
  
% The slope is smaller within FI of the band edges. There is approximately
% one extremum per coefficient. There are GridD grid points per extremum.
% On the average, there is one extremum every GridD points. Here we
% double the gridd density within 1.5*GridD points of the band edge.

% Fill in the frequency values for the piecewise linear transformation
% - Smaller step size within FI of the band edges
% - Exceptions: band edges that extend down to 0 or up to 0.5

Fw.f(1) = f(1);
Fw.f(2) = Fw.f(1);
Fw.f(3) = f(end);
Fw.f(4) = Fw.f(3);

if (f(1) > 0)
  Fw.f(2) = min (f(1) + FI, Fw.f(3));
end
if (f(end) < 0.5)
  Fw.f(3) = max (f(end) - FI, Fw.f(2));
end

% Remove zero length segments
I = (diff (Fw.f) <= 0);
Fw.f(I) = [];
R(I) = [];

% Fill in the piecewise linear mapping parameters
Nf = length (Fw.f);
if (Nf == 1)
  Fw.g(1) = Fw.f(1);
  Fw.S = 1;

else
  % Generate the g values
  DfR = diff (Fw.f) ./ R;
  a = cumsum ([0 DfR]) / sum ([0 DfR]);    % a = [0 ... 1]
  Fw.g = (1 - a) * Fw.f(1) + a * Fw.f(end);

  % Calculate the maximum slope df/dg: This slope is used to find the
  % uniformly spaced steps in g before being mapped through the piecewise
  % linear mapping.
  S = sum ([0 DfR]) / (Fw.f(end) - Fw.f(1));
  Fw.S = S * max(R);
end

return

%--------------------
function g = MM_warpInv (f, Fw)

if (length (Fw.f) == 1)
  g = Fw.g;
else
  g = MM_IntLin (Fw.f, Fw.g, f);
end

return

%--------------------
function f = MM_Warp (g, Fw)

if (length (Fw.f)== 1)
  f = Fw.f;
else
  f = MM_IntLin (Fw.g, Fw.f, g);
end

return

%---------------------
function yi = MM_IntLin (x, y, xi)
% Linear interpolation with a guarantee that if xi=x, then yi=y

[Nc, k] = histc (xi, x);
Nx = length (x);
k(k==Nx) = Nx - 1;

t = (xi - x(k)) ./ (x(k+1) - x(k));
yi = (1-t) .* y(k) + t .* y(k+1);

return

%---------------------
function GB = MM_GridVal (BSpec, FGrid)
% Given the input band specifications, this routine generates an equivalent
% specification on a dense array of frequency points. The intermediate
% values are generated by interpolating using a piecewise monotonic cubic
% function between the values of the input specifications.
%
% GB:    Interpolated values (band-by-band)
%
% BSpec: Band specifications
% FGrid: Frequency grid, band by band

% Calculate the desired response and the weight on the frequency grid
NBand = length (BSpec);
for (k = 1:NBand)

  % Monotonic cubic interpolation of the values, weights, and limits 
  f = BSpec(k).f;
  FG = FGrid{k};
  
  GB(k).FGrid = FG;
  GB(k).V = MM_IntMCI (f, BSpec(k).V, FG, BSpec(k).VI);
  GB(k).W = MM_IntMCI (f, BSpec(k).W, FG, BSpec(k).WI);
  GB(k).LL = MM_IntMCI (f, BSpec(k).LL, FG, BSpec(k).LLI);
  GB(k).UL = MM_IntMCI (f, BSpec(k).UL, FG, BSpec(k).ULI);

end

return

%---------------------
function VI = MM_IntMCI (f, V, FGrid, IT)
% Interpolate (f, V) to form (FGrid, VI)
% Monotonic cubic interpolation in done in a transformed domain, and the
% interpolated values are inverse transformed

if (length(f) == 1)
  VI = V;

else
  if (strcmp (IT, 'linear'))
    Fn = @(x) x;  % identity
    InvFn = Fn;
  elseif (strcmp (IT, 'log'))
    Fn = @log;
    InvFn = @exp;
  elseif (strcmp (IT, 'sqrt'));
    Fn = @sqrt;
    InvFn = @(x) x.^2;
  end
  VI = InvFn (IntMCI (f, Fn (V), FGrid));

end

return
