function a = MMintCof (x)
% This subroutine calculates a set of Lagrange interpolation coefficients
% given a set of abscissa values. The calculated coefficients are
% calculated as
%
%            N      1/2
%   a(k) = PROD  --------- .
%           i=1  x(k)-x(i)
%         (omit k)
%
% A scaling of all the a(k)'s will cancel in the interpolation formula. The
% reference below argues that a reasonable scaling factor for a(k) is
%   C^N, where C = 0.5 (max(x(i)) - min(x(i))).
% This is premised on a uniform distribution of x(i) values. Given that the
% range of the x values is +1 to -1, C should, by this argument, be about
% unity. However, for the filtering problem, the x(k) values will be
% clustered around band edges and so the average difference will tend be
% smaller than C. The code uses the factor 1/2 in the numerator is used to
% prevent the product from getting too small. In addition, because the x(k)
% are ordered, there is the possibility that we can have underflows or
% overflows in the partial products. The code scrambles the order of the
% coefficients to try to avoid this potential problem.
%
% See J.-P. Berrut and L. N. Trefethen, "Barycentric Lagrange
% Interpolation", SIAM Review, vol. 46, no. 3, pp. 501-517, 2004.
%
% ad: Array of Lagrange interpolation coefficients
%
% x: Array of (distinct) abscissa values

% $Id: MMintCof.m,v 1.2 2009/06/03 20:40:17 pkabal Exp $

% Calculate the interpolation coefficients
% The order is scrambled to try to preserve dynamic range.

persistent Nx I

% If Nx is empty or changes, form a new scrambled index vector
if (isempty (Nx))
  Nx = -1;
end
if (Nx ~= length (x))
  Nx = length (x);
  NSkip = 10;
  I = MM_ScrambleIndex (Nx, NSkip);
end
  
% Form the interpolation coefficient
a = zeros(1, Nx);
for (k = 1:Nx)
  dk = 2 * (x(k) - x);
  dk(dk == 0) = 1;
  a(k) = 1 / prod (dk(I));
end

return

%--------------------
function I = MM_ScrambleIndex (N, NSkip)

% Create a scrambled index array
% - Fill array index with 1:Nx followed by zeros to a multiple of NSkip
% - Make it M x NSkip
% - Transpose it
% - Make it a vector
% - Kill the zero-valued fill elements

M = ceil (N / NSkip);
NI = M * NSkip;

I = reshape ([1:N zeros(1,NI-N)], M, NSkip)';
I = reshape (I, 1, NI);
I(I == 0) = [];

return
