function y = MMintVal (x, P)
% This function calculates the interpolated value of a polynomial using the
% Lagrange interpolation formula in barycentric form. The interpolation
% formula is as follows.
%
%             N  a(m) y(m)
%            SUM ---------
%            m=1  x-x(m)                   N        c
%   Val(x) = -------------,  where a(m) = PROD  ---------,
%               N   a(k)                  i=1   x(m)-x(i)
%              SUM ------               (omit m)
%              k=1 x-x(k)
%
% This formula is equivalent to the conventional Lagrange interpolation
% formula which can be expressed as the sum of N polynomials,
%
%             N                              a(m)   N
%   Val(x) = SUM y(m) L(m,x), where L(m,x) = ---- PROD (x-x(i)) .
%            m=1                               c   i=1
%                                                (omit m)
%
% The equivalence of the two forms can be demonstrated by multiplying
% numerator and denominator of the first formulation by PROD (x-x(i)). The
% numerator is then in the form of the conventional Lagrange interpolation
% formula, while the denominator is the Lagrange interpolation of a
% constant.
%
% The barycentric form has the advantage that the computational complexity
% is proportional to N once the interpolation coefficients have been
% calculated. A disadvantage is that the result is 0/0 at the sample
% points.
%
% y: Interpolated value
% x: Argument value. The polynomial is evaluated at this point. This value
%    should not correspond to any of the abscissa values, lest a divide by
%    zero occur.
% P: Structure containing the array of Lagrange interpolation coefficients
%    and the corresponding abscissa and ordinate values. These values
%    together specify the polynomial that passes through the points
%    specified.

% $Id: MMintVal.m,v 1.2 2009/06/03 20:19:22 pkabal Exp $

d = P.a ./ (x - P.x);
y = sum (d .* P.y) / sum (d);

return
