function [devs, ier] = MMmmDev (ad, G, Ext, EType)
% This routine calculates the deviation for a minimax approximation. The
% input is a set of desired values, weights and limits at NExt points. This
% routine solves for the deviation that allows for a NExt-1 term polynomial
% to alternate about the desired values, with equal (weighted) deviation at
% the given abscissa values.
%
% devs: Deviation at the design points
% ier: Error parameter.
%     0 - No error
%     3 - Constraints too tight for the given filter order
%
% ad: Array of Lagrange interpolation coefficients
% G:  Structure with the arrays of filter specifications on the frequency
%     grid.
% Ext: Indices of the alternation points.
% EType: Type of extremum (NExt values). EType(i) takes on one of the
%     following values,
%     -2 - Lower constraint limit
%     -1 - Lower ripple
%     +1 - Upper ripple
%     +2 - Upper constraint limit

% $Id: MMmmDev.m,v 1.2 2009/06/03 20:17:20 pkabal Exp $

% Calculate the deviation:
% Given a set of NExt distinct points, an NExt-1 coefficient polynomial can
% give an error curve which exhibits equal amplitude alternations (in a
% weighted sense) about the desired value. In such a formulation, the free
% parameters are the NExt-1 coefficients and the deviation giving a total
% of NExt linear equations. The strategy is to solve for the unique value
% of the deviation which allows for this alternation of the error, without
% solving directly for the coefficients of the polynomial. The deviation is
% found using Cramer's rule. The determinants in this formulation can be
% expressed in terms of the determinant of a Vandermonde matrix. The
% expression for the deviation is expressed in terms of the Lagrange
% coefficients for an NExt value interpolation.

E_LDEV = -1;
E_UDEV = 1;
E_LLIM = -2;
E_ULIM = 2;

NExt = length (ad);
num = 0;
den = 0;
for (i = 1:NExt)
  k = Ext(i);
  if (EType(i) == E_LDEV)
    num = num + ad(i) * G.V(k);
    den = den + ad(i) / G.W(k);

  elseif (EType(i) == E_UDEV)
    num = num + ad(i) * G.V(k);
    den = den - ad(i) / G.W(k);

  elseif (EType(i) == E_LLIM)
    num = num + ad(i) * G.LL(k);

  elseif (EType(i) == E_ULIM)
    num = num + ad(i) * G.UL(k);

  end
end

if (den == 0)
  if (nargout == 2)
    disp ('DFiltInt - Constraints too tight');
  end
  ier = 3;
  devs = Inf;
else
  ier = 0;
  devs = num / den;
end

return
