function Gc = MMmodSpec (FType, FCase, GB)
% This routine modifies the specifications of certain types of filters.
% The specifications are given in terms of a desired value, weight and
% constraints at given frequencies. For receiving filters, the
% specifications are modified to include an x/sin(x) factor. For
% differentiators, the specifications are changed from a desired slope to
% a desired value.
%
% This routine also transforms the specifications for the design of the
% different cases (odd/even symmetry, odd/even number of coefficients) into
% a canonical form.

% $Id: MMmodSpec.m,v 1.3 2009/07/02 21:33:15 pkabal Exp $

Gmod = MM_ModSpec (FType, GB);
Gc = MM_CanonSpec (FCase, Gmod);

return

%--------------------
function Gmod = MM_ModSpec (FType, GB)
% This routine modifies the specifications of receiving filters and
% differentiators.

% F_BPF = 1;
F_REC = 2;
F_DIF = 3;
% F_HIL = 4;

% Frequency points
FGrid = GB.FGrid;
Gmod.FGrid = FGrid;

switch FType
  case F_DIF
    Gmod.V = FGrid .* GB.V;
    Gmod.W = GB.W ./ FGrid;
    Gmod.LL = FGrid .* GB.LL;
    Gmod.UL = FGrid .* GB.UL;
  case F_REC
    Isinc = 1 ./ sinc (FGrid);
    Gmod.V = Isinc .* GB.V;
    Gmod.W = GB.W;
    Gmod.LL = Isinc .* GB.LL;
    Gmod.UL = Isinc .* GB.UL;
  otherwise
    Gmod.V = GB.V;
    Gmod.W = GB.W;
    Gmod.LL = GB.LL;
    Gmod.UL = GB.UL;
end

return

%--------------------
function Gc = MM_CanonSpec (FCase, G)
% Given the specifications of a linear phase FIR filter on a frequency
% grid, this routine transforms the specifications for the design of the
% different cases (odd/even symmetry, odd/even number of coefficients) into
% a canonical form.
%
% There are four cases to be considered for a linear phase filter,
%          N-1       -n 
%   H(z) = SUM h(n) z  .
%          n=0
% Case     symmetry       even/odd
%  1)    h(n) = h(N-1-n)    N odd     bandpass filter
%  2)    h(n) = h(N-1-n)    N even    bandpass filter
%  3)    h(n) = -h(N-1-n)   N odd     differentiator/Hilbert transform
%  4)    h(n) = -h(N-1-n)   N even    differentiator/Hilbert transform
%
% The frequency response, H(w) = H(exp(jw)), can be written in the
% following form for each of the cases,
%                               M-1
%   H(w) = exp(jw(N-1)/2) P'(w) SUM a(n) cos(wn) .
%                               n=0
%
% Case     P'(w)       No. coef. (M)
%  1)        1          (N-1)/2+1
%  2)    2 cos(w/2)      N/2
%  3)    2j sin(w)      (N-3)/2+1
%  4)    2j sin(w/2)     N/2
% The given specifications will be converted to an equivalent specification
% for approximation by a sum of cosines. The phase factors in H(w) do not
% enter into the magnitude approximation. However, the fixed factor P'(w)
% must be absorbed into the desired value, constraint values, and weighting.
%
% In addition, this routine converts the frequency grid values (f from
% 0 to 1/2) to values of x = cos(2Pi f) (x from +1 to -1).

% $Id: MMmodSpec.m,v 1.3 2009/07/02 21:33:15 pkabal Exp $

% Let H(z) be a linear phase FIR filter. Then H(z) is symmetric or
% anti-symmetric,
%   H(z) = +/- z^(-(N-1)) H(1/z).
% For z=1 and z=-1, H(z) = H(1/z) and z^(-(N-1)) will be +1 or -1.
% Then if we write H(z) in the following form, we can identify the
% cases for which H(z) has a root at +1 and/or -1,
%   H(z) = 0.5 [ H(z) +/- z^(-(N-1)) H(1/z) ].
% If we do find roots, then H(z) can be written as
%   H(z) = P(z) B(z).
% If we look at the symmetries of H(z) and P(z), we can ascertain the
% symmetry of B(z), which will turn out to be symmetric.
%
% case 1: N odd, symmetric
%     H(z) = B(z),
%   where B(z) is symmetric with an odd number of terms (K=N).
% case 2: N even, symmetric
%     H(z) = 0.5 [ H(z) + z^(-(N-1)) H(1/z) ].
%   This is zero for z=-1. Then H(z) can be written as
%     H(z) = (1 + z^(-1)) B(z),
%   where B(z) is symmetric with an odd number of terms (K=N-1).
% case 3: N odd, anti-symmetric
%     H(z) = 0.5 [ H(z) - z^(-(N-1)) H(1/z) ].
%   This has roots at z=1 and z=-1. Then H(z) can be written as
%     H(z) = (1 - z^(-2)) B(z),
%   where B(z) is symmetric with an odd number of terms (K=N-2).
% case 4: N even, anti-symmetric
%     H(z) = 0.5 [ H(z) - z^(-(N-1)) H(1/z) ].
%   This has a root at z=1. Then H(z) can be written as
%     H(z) = (1 - z^(-1)) B(z),
%   where B(z) is symmetric with an odd number of terms (K=N-1).
%
% In all cases, H(z) can be written as
%     H(z) = P(z) B(z)
%          = P(z) z^(-(K-1)/2) B'(z)
%          = z^(-(N-1)/2) P'(z) B'(z).
%
% The frequency response of P'(z) is used to modify the specifications
% on H(z) to equivalent specifications for the design of B'(z).
%   Case        P'(z)           P'(w)
%     1          1               1
%     2  z^(1/2) + z^(-1/2)   2 cos(w/2)
%     3        z - z^(-1)      2 sin(w)
%     4  z^(1/2) - z^(-1/2)   2 sin(w/2)
%
% B(z) is symmetric with an odd number of coefficients. B'(z) is symmetric
% about 0. Then, letting the number of coefficients be K = 2M-1,
%                    M-1
%   B'(z) = b[M-1] + SUM [ b[M-1-k] z^k + b[M-1+k] z^(-k) ]
%                    k=1
%                    M-1
%         = b[M-1] + SUM b[M-1-k] [ z^k + z^(-k) ]
%                    k=1
%                      M-1
%   B'(w) = b[M-1] + 2 SUM b[M-1-k] cos(wk)
%                      k=1
%           M-1
%         = SUM a[k] cos(wk)
%           k=0
%   a[0] = b[M-1]                       b[M-1] = a[0]
%   a[k] = 2 b[M-1-k]   1 <= k <= M-1   b[k] = 0.5 a[M-1-k] = b[2M-2-k]

FGrid = G.FGrid;
switch FCase
  case 1
    ch = 1;
  case 2
    ch = 2 * cos (pi * FGrid);
  case 3
    ch = 2 * sin (2 * pi * Fgrid);
  case 4
    ch = 2 * sin (pi * FGrid);
end

% Transform the specifications
Gc.x = cos (2 * pi * FGrid);
Gc.V = G.V ./ ch;
Gc.W = ch .* G.W;
Gc.LL = G.LL ./ ch;   % Assumes ch is positive
Gc.UL = G.UL ./ ch;

% Check for frequency values too closely spaced after transformation
% (x decreases with index)
if (any (diff (Gc.x) >= 0))
  error ('DFiltFIR: Grid points too closely spaced');
end

return
