function d = SlopeMC (x, y)
% Slopes at each point for use with polynomial interpolation using a
% piecewise monotonic cubic interpolant.

% $Id: SlopeMC.m,v 1.4 2009/06/05 15:49:10 pkabal Exp $

dx = diff(x);
dy = diff(y);

% Check for increasing X values
if (any(dx) < 0)
  error ('SlopeMC: X values not increasing');
end

% Slopes of line segments
% S(i) is the slope joining [x(i-1) y(i-1)] and [x(i), y(i)]
S = dy ./ dx;   % Slopes of line segments

% Initialize slopes at points to zero
d = zeros(size(y));

%  Special case n=2, use linear interpolation
N = length(x);
if (N == 2)
  d(1) = S(1);
  d(2) = S(1);
  return
end

% Find all points which have line segment slopes on either side of the
% same sign. S(i) is the slope between points i and i+1).
%        left of point     right of point
I = find(sign(S(1:N-2)) .* sign(S(2:N-1)) > 0);

% Use Brodlie modification of Butland formula (for same sign slopes)
%
%             Sl Sr                  1       xl-x0
%    d = --------------- , where a = - (1 + -------) .
%        a Sl + (1-a) Sr             3      xl - xr
%
% This formula gives the harmonic mean of the slopes when dx(i-1)=dx(i).
% The factor a modifies the formula to take into account the relative
% spacing of the values (a varies from 1/3 to 2/3). This formula satisfies
% the following properties.
% (a) the formula is invariant to an exchange of the the left and right
%     points,
% (b) min(Sl,Sr) <= d <= max(Sl,Sr),
% (c) |d| <= 3 min(|Sl|,|Sr|).  This is a sufficient condition for
%       monotonicity if d is calculated in this way for neighboring points.
%
% The following calculation is designed to avoid overflows, but is
% equivalent to the above formula. Note that the product Sl Sr is
% positive when this formula is applied.
%         Sl Sr              +/- Smin
%   ---------------  = -------------------------
%   a Sl + (1-a) Sr    a Sl/Smax + (1-a) Sr/Smax

sdx = dx(I) + dx(I+1);             % Left and right of I+1
a0 = (dx(I)   + sdx) ./ (3 * sdx);
a1 = (dx(I+1) + sdx) ./ (3 * sdx);

Smax = max(abs(S(I)), abs(S(I+1)));
Smin = min(abs(S(I)), abs(S(I+1)));

d(I+1) = Smin ./ ( a0 .* (S(I) ./ Smax) + a1 .* (S(I+1) ./ Smax) );

% Slopes at the end points
% The calculated derivative depends on ratios of the segment lengths and
% the slopes, so the same equation applies at either end
d(1) = SlopeEndPMC(dx(1), dx(2), S(1), S(2));
d(N) = SlopeEndPMC(dx(N-1), dx(N-2), S(N-1), S(N-2));

return

%---------------
function d = SlopeEndPMC (dxn, dxf, Sn, Sf)
% dxn is the adjacent segment length, dxf is the far side segment length
% Sn is the adjacent slope, Sf is the slope on the far side

% Set the derivative using a non-centered three-point formula, adjusted to
% be shape-preserving
%                                           dxn
%    d = a Sn + (1-a) Sf , where a = 1 + --------- .
%                                        dxn + dxf
%
% The factor a modifies the formula to take into account the relative
% spacing of the values (a varies from 2 to 1). a=1.5 for equal segment
% lengths. If the slopes are equal, then the value of a does not affect
% the slope.
Ds = dxf + dxn;
d = Sn * ((Ds + dxn) / Ds) - Sf * (dxn / Ds);

% Modify the value if necessary to preserve monotonicity
if (sign(d) ~= sign(Sn))
  d = 0;
elseif (sign(Sn) ~= sign(Sf) && abs(d) > abs(3.0 * Sn))
  d = 3 * Sn;
end

return
