function Example1

% $Id: Example1.m,v 1.5 2009/06/17 22:21:37 pkabal Exp $

addpath ('..');

% Design of an Anti-aliasing Filter
% 
% Consider the design of an anti-aliasing filter for filtering prior to
% converting between a sampling rate of 48 kHz and a sampling rate of 8
% kHz. Such a filter could for instance be used in downsampling the output
% of a Digital Audio Tape (DAT) recorder. In such an application, the input
% signal is sampled with a resolution of 16 bits resulting in a sine-wave
% Signal-to-Noise ratio of nearly 100 dB.
% 
% Ideally the anti-aliasing filter should be a brick wall filter cutting
% off at 4 kHz. A practical filter requires a finite width transition band.
% The stopband should start at 4 kHz to avoid aliasing. This means the
% passband edge must be below 4 kHz, inevitably removing some of the "high"
% frequencies in the 8 kHz signal. Consider using a filter with cutoff at
% 3450 Hz.  We will use 255 coefficients.
% 
% The first attempt uses a weight of one in the passband and a weight of
% ten in the stopband.

Fs = 48000;

clear B
[B(1:2).Freq]   = deal([0 3450], [4000, 24000]);
[B(1:2).Value]  = deal(1, 0);
[B(1:2).Weight] = deal(1, 10);
h = DFiltFIR(255, B, 'bpf', Fs);
PlotFilter(h, 1, Fs);

% This filter is a good start, with 0.05 dB passband ripple and a minimum
% 64 dB stopband attenuation.
% 
% We can improve on this design by applying a weighting which changes
% across a band. Basically, the critical region is near the passband and
% stopband edges. We can increase the attenuation far from the stopband
% edge with only a small effect on the attenuation at the stopband edge.
% Here we will specify increased attenuation at frequencies above 6 kHz.
% We would like to push this attenuation down to near 90 dB. This means a
% 30 dB increase or about a 30-fold increase in weight.

clear B
[B(1:2).Freq]   = deal([0 3450], [4000, 6000, 24000]);
[B(1:2).Value]  = deal(1, 0);
[B(1:2).Weight] = deal(1, [10 100 300]);
h = DFiltFIR(255, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs, [-90, 10]);

% We have pushed the extra attenuation quite close to the band edge and
% have paid a penalty - the passband ripple is now 0.08 dB.
% 
% Such a filter might benefit from a dc constraint. The dc response can
% be constrained to be unity, so that constant values in the input are
% preserved at the output of the filter. In addition, we can taper the
% attenuation in the passband so that the low frequencies suffer less from
% the passband ripple than higher frequencies. We require the ripple near
% zero to be 1/5 of the ripple at the passband edge.
%
% The dc constraint is imposed by setting the upper and lower limits to
% unity at the lower edge of the first band (dc). The rest of the first
% band has infinite limits (no limits). An interpolation of these values
% results in infinte limits except at dc.

clear B
[B(1:2).Freq]   = deal([0 3450], [4000, 6000, 24000]);
[B(1:2).Value]  = deal(1, 0);
[B(1:2).Weight] = deal([5 1], [10 100 300]);
B(1).LLimit = [1 -Inf];
B(1).ULimit = [1 Inf];
h = DFiltFIR(255, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs, [-90, 10]);

% The ripple at the passband edge has deteriorated slightly, but the ripple
% in most of the passband has decreased significantly.
% 
% To increase the passband size, we will have to also increase the
% number of coefficients. Consider setting the passband edge to 3700 and
% increasing the number of coefficients to 511.  After a number of
% iterations on the weights, the following design seems to be a reasonable
% compromise.

clear B
[B(1:2).Freq]   = deal([0 3000 3700], [4000, 6000, 24000]);
[B(1:2).Value]  = deal(1, 0);
[B(1:2).Weight] = deal([15 5 1], [1.5 20 100]);
B(1).LLimit = [1 -Inf -Inf];
B(1).ULimit = [1 Inf Inf];
h = DFiltFIR(511, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs, [-90, 10]);

return
