function Example2

% $Id: Example2.m,v 1.1 2009/06/16 17:19:38 pkabal Exp $

addpath ('..');

% The Problem of Resonances in the Transition Bands

% Consider the following design from [1]. It is a stopband filter with two
% transition bands. The following code shows the design values.
% 
% [1] G. Cortelazzo, M. R. Lightner and W. K. Jenkins,
%     "An Alternate Technique for Min-Max Design of Multiband Finite
%     Impulse Response Digital Filters",
%     Circuits, Systems and Signal Processing, pp. 285-309, Vol. 2, No. 3,
%     1983.

Fs = 1;

clear B
[B(1:3).Freq]   = deal([0 0.143759973], [0.16533943, 0.37032451], ...
                       [0.41679744 0.5]);
[B(1:3).Value]  = deal(1, 0, 1);
[B(1:3).Weight] = deal(0.04588809, 1, 0.03853275);
h = DFiltFIR(75, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs);

%            Freq.      Value     Weight          Limits      Deviation  Dev dB
% Band  1: 0.0000          1     0.0459      ----       ----      0.072    0.65
%          0.1438          1     0.0459      ----       ----      0.072    0.65
% Band  2: 0.1653          0          1      ----       ----     0.0033  -49.61
%          0.3703          0          1      ----       ----     0.0033  -49.61
% Band  3: 0.4168          1     0.0385      ----       ----      0.086    0.78
%          0.5000          1     0.0385      ----       ----      0.086    0.78

% This filter exhibits a non-monotonic transition in the transition from
% the stopband to the second passband. The transition band response rises
% to a peak well above 0 dB near frequency 0.4, before settling down to
% near unity at the passband edge (0.417). The band edge extremum in the
% passband is a downward one (as would normally occur at the passband
% edge), but the slope is zero, and the response rises as we go down in
% frequency into the transition band. This kind of "resonance" does not
% occur in simple low-pass or highpass filters. It occurs only in some
% multiband designs.

% First let us specify the values in a new band which overlays the second
% transition band. The desired value will rise from 0 to 1 in this new
% band, but the weight will be extremely small. First let us verify that
% just specifying this new band does not substantially change the design.

clear B
[B(1:4).Freq]   = deal([0 0.143759973], [0.16533943, 0.37032451], ...
                       [0.370326, 0.416796], [0.41679744 0.5]);
[B(1:4).Value]  = deal(1, 0, [0, 1], 1);
[B(1:4).Weight] = deal(0.04588809, 1, 1e-6, 0.03853275);
h = DFiltFIR(75, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs);

%            Freq.      Value     Weight          Limits      Deviation  Dev dB
% Band  1: 0.0000          1     0.0459      ----       ----      0.072    0.65
%          0.1438          1     0.0459      ----       ----      0.072    0.65
% Band  2: 0.1653          0          1      ----       ----     0.0033  -49.61
%          0.3703          0          1      ----       ----     0.0033  -49.61
% Band  3: 0.3703          0     1e-006      ----       ----   3.3e+003   70.39
%          0.4168          1     1e-006      ----       ----   3.3e+003   70.39
% Band  4: 0.4168          1     0.0385      ----       ----      0.086    0.78
%          0.5000          1     0.0385      ----       ----      0.086    0.78

% This filter is substantially the same as without the extra band. Now let
% us add constraints to the transition band. This band is now constrained
% to have values betweeen 0 and 1.

clear B
[B(1:4).Freq]   = deal([0 0.143759973], [0.16533943, 0.37032451], ...
                       [0.370326, 0.416796], [0.41679744 0.5]);
[B(1:4).Value]  = deal(1, 0, [0, 1], 1);
[B(1:4).Weight] = deal(0.04588809, 1, 1e-6, 0.03853275);
B(3).LLimit = 0;
B(3).ULimit = 1;
h = DFiltFIR(75, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs);

%            Freq.      Value     Weight          Limits      Deviation  Dev dB
% Band  1: 0.0000          1     0.0459      ----       ----      0.074    0.67
%          0.1438          1     0.0459      ----       ----      0.074    0.67
% Band  2: 0.1653          0          1      ----       ----     0.0034  -49.42
%          0.3703          0          1      ----       ----     0.0034  -49.42
% Band  3: 0.3703          0     1e-006          0          1         1    0.00
%          0.4168          1     1e-006          0          1         1    0.00
% Band  4: 0.4168          1     0.0385      ----       ----      0.088    0.80
%          0.5000          1     0.0385      ----       ----      0.088    0.80

% The new filter does not go above unity in the "transition" band. Note
% however, the slight increase in passband and stopband ripple. This is the
% price paid for the constraint. The plot of the frequency response reveals
% that the resonance is still present at f=0.4, but the peak value is now
% unity. This effectively means that the passband really extends to some
% frequency below 0.4.
% 
% Noting that the passband effectively goes below the actual band edge,
% suggests that we can move the passband edge so that the resonance becomes
% a ripple in the passband instead of the transition band. In a final
% filter design, we redefine the upper passband edge to 0.398, while
% leaving all other frequencies the same as in the original design (no
% constraints, no extra band).

clear B
[B(1:3).Freq]   = deal([0 0.143759973], [0.16533943, 0.37032451], ...
                       [0.398 0.5]);
[B(1:3).Value]  = deal(1, 0, 1);
[B(1:3).Weight] = deal(0.04588809, 1, 0.03853275);
h = DFiltFIR(75, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs);

%            Freq.      Value     Weight          Limits      Deviation  Dev dB
% Band  1: 0.0000          1     0.0459      ----       ----      0.073    0.66
%          0.1438          1     0.0459      ----       ----      0.073    0.66
% Band  2: 0.1653          0          1      ----       ----     0.0034  -49.44
%          0.3703          0          1      ----       ----     0.0034  -49.44
% Band  3: 0.3980          1     0.0385      ----       ----      0.088    0.80
%          0.5000          1     0.0385      ----       ----      0.088    0.80

% A ripple still occurs at around f=0.4, but this is now considered part of
% the passband. This "overshoot" is the same as the ripple amplitude in the
% rest of the passband. There is also a zero in the transition band at the
% lower end of the transition band - this effectively extends the stopband.
%
% As a final note, a number of other options could have been considered.
% The order of the filter could have been reduced. In this case reducing
% the order to 73, stops an overshoot in the transition band, though there
% is an extra zero at around f=0.4. A true monotonic transition can be
% forced by carefully choosing the constraint values - in particular, the
% constraint values should rise from a low value (0.5 works) to a value
% equal to one minus the deviation in the passband at the passband edge
% (say 0.913).

return
