function Example3

% $Id: Example3.m,v 1.2 2009/06/22 20:22:04 pkabal Exp $

addpath ('..');

% Intermediate Reference System Filter

% CCITT Recommendation P.48 specifies intermediate reference system
% filters. These are filters that can be used to filter speech signals
% before and after transmission. Here we consider the design of a sending
% IRS filter. This filter can be used to model the filtering applied to
% speech signals before processing in a speech coding system.
% 
% The IRS specifications are given as tabulated upper and lower bounds on
% the frequency response of the filter. The actual bounds are the straight
% line segments on a dB - log frequency plot which join the tabulated
% values. Furthermore tabulated nominal values are also given, but without
% a rule as to how intermediate nominal values should be determined.
% 
%    freq   lower limit  nominal  upper limit  
%     Hz       dB           dB        dB
% 
%     100                -45.8      -41
%     125                -36.1
%     160                -25.6
%   < 200   -infty
%     200    -21         -19.2      -16
%     250                -14.3
%     300                -11.3
%     315                -10.8
%     400    -11          -8.4       -6
%     500                 -6.9
%     600                 -6.3
%     630                 -6.1
%     800                 -4.9
%    1000                 -3.7
%    1250                 -2.3
%    1600                 -0.6
%    2000                  0.3
%    3000     -1           1.5
%    3150                  1.8
%    3400     -4           ---      +6
%  > 3400    -infty
%    3500                 -7.3
%    3600                           +4
%    4000                -37.2
%    5000                -52.2
%    6000                          -60
%    6300                -73.6
%    8000                -90.0
% 
% The filter will be designed for 8 kHz sampling, so that only values up to
% 4 kHz will be of interest. This design would seem to be a candidate for
% the use of limits in the input to DFiltFIR. However, one normally uses
% limits when one expects the approximation to actually hit the limits.
% Here we want a filter which fits well inside the limits. We will use
% the P.48 limit curves to set the weights, so that the excursion of the
% approximation about the nominal values will depend on the distance
% between to the nearest limit curve.
% 
% As a first step we need to be able to sample the limit curves. The P.48
% limit curves are linear on a log-log scale. To sample these we have to
% write a short program to sample the curves. These values are then used to
% design the filter.

dBVInv = @(x) 10.^(x/20); 

Freq = [100 125 160 200 250 300 315 400 500 600 ...
        630 800 1000 1250 1600 2000 3000 3150 3500 4000];
ValuedB = [-45.8 -36.1 -25.6 -19.2 -14.3 -11.3 -10.8 -8.4 -6.9 -6.3 ...
           -6.1 -4.9 -3.7 -2.3 -0.6 0.3 1.5 1.8 -7.3 -37.2];
FreqLL = [100 200 400 3000 3400 4000];
LLimdB = [-Inf -21 -11 -1 -4 -Inf];
FreqUL = [100 200 400 3000 3600 6000];
ULimdB = [-41 -16 -6 +6 +4 -60];

% Determine the limits at the Freq points
LLimdBint = interp1(log(FreqLL), LLimdB, log(Freq));
I = find(Freq < 200 | Freq > 3400);
LLimdBint(I) = -Inf;
ULimdBint = interp1(log(FreqUL), ULimdB, log(Freq));
figure;
semilogx(Freq, max(LLimdBint, -1E10), '--g', Freq, ValuedB, 'ro-', ...
         Freq, ULimdBint, '--g');
axis([100, 4000, -60, 10]);
xlabel('Frequency (Hz)');
ylabel('Amplitude (dB)');
title('Plot of limits and desired values vs log frequency');

% Now convert dB values to linear values
% Find the distance to the limit
% Make the weight inversely proportional to the distance
Value = dBVInv(ValuedB);
DevMin = min(Value - dBVInv(LLimdBint), dBVInv(ULimdBint - Value));
Weight = 1 ./ DevMin;

% We will use a filter length of 151. Smaller lengths as low as 33 will
% result in filters whose responses fall within the specifications, but
% which exhibit relatively large ripples about the desired values.

clear B
Fs = 8000;
B.Freq  = Freq;
B.Value = Value;
B.Weight = Weight;
h = DFiltFIR(151, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs);
hold on;
plot(Freq, max(LLimdBint, -1E10), '--g', Freq, ValuedB, 'ro', ...
     Freq, ULimdBint, '--g');
hold off;
xlabel('Frequency (Hz)');
ylabel('Amplitude (dB)');
title('Plot of limits, desired values, (first) filter response');

% The filter specification does not cover frequencies below 100 Hz. Without
% any specification below 100 Hz, the attenuation at dc is about 30 dB.
% This dc attenuation is less than the required minimum attenuation at
% 100 Hz (41 dB).
%
% Noting that we meet the specifications at 200 Hz with the weight at that
% frequency, we use that same weight for all frequencies below 200.

clear B
B.Freq = [0, Freq];
B.Value = [0, Value];
B. Weight = [Weight(1), Weight];
h = DFiltFIR(151, B, 'bpf', Fs);
figure;
PlotFilter(h, 1, Fs);
hold on;
plot(Freq, max(LLimdBint, -1E10), '--g', Freq, ValuedB, 'ro', ...
     Freq, ULimdBint, '--g');
hold off;
xlabel('Frequency (Hz)');
ylabel('Amplitude (dB)');
title('Plot of limits, desired values, (second) filter response');

% Compare this design with the filter used by the TIA and which also
% appears in the ITU-T Software Tools Library. The TIA filter shows larger
% ripples between the desired points.
[hTIA, b] = ReadFilter('TIA_IRS_FIR_8k.cof');
figure;
PlotFilter(hTIA, b, Fs);
hold on;
plot(Freq, max(LLimdBint, -1E10), '--g', Freq, ValuedB, 'ro', ...
     Freq, ULimdBint, '--g');
hold off;
xlabel('Frequency (Hz)');
ylabel('Amplitude (dB)');
title('Plot of limits, desired values, TIA filter response');

return
